<?php
/**
 * @file
 * - Form alters for deployer.
 * - Node form validation (1 deploy per env at the same time).
 */

/**
 * Implements hook_form_alter().
 */
function deployer_form_alter(&$form, &$form_state, $form_id) {

  // Warn for removal of all projects / environments / snapshots related
  // to this client.
  if ($form_id == 'node_delete_confirm') {
    switch ($form['#node']->type) {
      case 'client':
        $warning = t('Please not that all underlying projects and environments will be removed from dashboard as well, including backups!');
        break;

      case 'project':
        $warning = t('Please not that all underlying environments will be removed from dashboard as well, including backups!');
        break;
    }

    if (isset($warning)) {
      $form['deployer_warning'] = array(
        '#type' => 'markup',
        '#markup' => theme('html_tag', array(
          'element' => array(
            '#tag' => 'div',
            '#value' => $warning,
          ),
        )),
        '#weight' => 0,
      );
    }
  }

  // Alter deploy.
  if (isset($form['type']['#value']) && $form['type']['#value'] == 'deploy') {

    // Restore.
    if (isset($_GET['restore']) && $_GET['restore'] == 1) {

      // Provide title.
      $form['title']['#default_value'] = 'restore_' . uniqid();

      // Hide title field.
      $form['title']['#type'] = 'hidden';

      // Page title.
      drupal_set_title(t('Restore a snapshot'));

      // '--restore--' is symbol for making a backup only, instead of deploy.
      $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#default_value'] = '--restore--';
      $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';

      $form['field_revert_features']['#type'] = 'hidden';
      $form['field_drush_update']['#type'] = 'hidden';
      $form['field_l10n_update']['#type'] = 'hidden';

      // Set restore id.
      if (isset($_GET['id']) && !empty($_GET['id']) && is_numeric($_GET['id']) && $node = node_load($_GET['id'])) {
        $form['field_restore_id'][LANGUAGE_NONE][0]['value']['#default_value'] = $_GET['id'];

        $env = field_get_items('node', $node, 'field_environment_id');
        $env = $env[0]['target_id'];
        $env = node_load($env);

        $project = field_get_items('node', $env, 'field_project');
        $project_id = $project[0]['target_id'];

        $query = db_select('field_data_field_project', 'p');
        $query->condition('p.field_project_target_id', $project_id);
        $query->fields('p', array('entity_id'));
        $results = $query->execute();

        $options = array();
        $options['_none'] = t('- Select a value -');
        foreach ($results as $row) {
          $envnode = node_load($row->entity_id);
          $options[$row->entity_id] = $envnode->title;
        }
        $form['field_environment_id'][LANGUAGE_NONE]['#options'] = $options;

      }

      module_load_include('inc', 'deployer', 'deployer_deploy_lists');
      $snapshot = deployer_download_snapshot();

      $form['snapshot'] = array(
        '#markup' => drupal_render($snapshot),
        '#weight' => 0,
      );

    }

    // Deploy / backup.
    elseif (isset($_GET['env']) && !empty($_GET['env']) && is_numeric($_GET['env']) && $env = node_load($_GET['env'])) {
      $field_project = field_get_items('node', $env, 'field_project');
      if (isset($field_project[0]['target_id']) && $project = node_load($field_project[0]['target_id'])) {

        $field_connection_type = field_get_items('node', $env, 'field_connection_type');
        if (empty($field_connection_type)) {
          // Assume SSH if not provided.
          $connection_type = 'ssh';
        }
        else {
          $connection_type = $field_connection_type[0]['value'];
        }

        $backup = FALSE;
        if (isset($_GET['backup']) && $_GET['backup'] == 1) {
          $backup = TRUE;
          // Provide title.
          $form['title']['#default_value'] = 'backup_' . uniqid();

          // Page title.
          drupal_set_title(t('Backup an environment'));

          // Hide drush field_group.
          if (isset($form['#fieldgroups']['group_drush'])) {
            foreach ($form['#fieldgroups']['group_drush']->children as $child) {
              unset($form['#group_children'][$child]);
            }
            unset($form['#fieldgroups']['group_drush']);
            unset($form['#groups']['group_drush']);
          }
        }
        else {
          // Provide title.
          $form['title']['#default_value'] = 'deploy_' . uniqid();

          // Page title.
          drupal_set_title(t('Deploy project to an environment'));
        }

        // Hide title field.
        $form['title']['#type'] = 'hidden';

        // Hide restore field_group.
        if (isset($form['#fieldgroups']['group_restore'])) {
          foreach ($form['#fieldgroups']['group_restore']->children as $child) {
            unset($form['#group_children'][$child]);
          }
          unset($form['#fieldgroups']['group_restore']);
          unset($form['#groups']['group_restore']);
        }

        // Create nice hidden environment-field and description.
        $form['field_environment_id'][LANGUAGE_NONE]['#default_value'] = $_GET['env'];
        $form['field_environment_id'][LANGUAGE_NONE]['#type'] = 'hidden';

        $field_text = '<div class="environment"><span class="label">Environment:</span> <span>' . $env->title . '</span> for project <span>' . $project->title . '</span></div>' . "\n";

        if (!$backup) {
          $field_text .= '<div class="code">Branch <span>' . $env->field_code_branch[LANGUAGE_NONE][0]['safe_value'] . '</span> on project <span>' . deployer_repo_link($project->field_code_project[LANGUAGE_NONE][0]['code_project'], 'short') . '</span></div>' . "\n";
        }
        $form['field_environment_id'][LANGUAGE_NONE]['#suffix'] = $field_text;

        // Commit tags.
        if ($backup) {
          // '--dummy--' is symbol for making a backup only, instead of deploy.
          $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#default_value'] = '--dummy--';
          $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';
        }
        else {
          // Get tag options from option provider with select box,
          // instead of text field.
          $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#type'] = 'select';
          $form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#options'] = deployer_tags_list($env->field_project[LANGUAGE_NONE][0]['target_id'], $_GET['env']);
          unset($form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#maxlength']);
          unset($form['field_commit_tag'][LANGUAGE_NONE][0]['value']['#size']);
        }

        // Make drush options hidden in case of backup of ftp connection.
        if ($backup || $connection_type == 'ftp') {
          $form['field_revert_features']['#type'] = 'hidden';
          $form['field_drush_clear_cache']['#type'] = 'hidden';
          $form['field_drush_update']['#type'] = 'hidden';
          $form['field_l10n_update']['#type'] = 'hidden';
          $form['field_reset_all_passwords']['#type'] = 'hidden';
          $form['field_search_api_reindex']['#type'] = 'hidden';
        }

        // Make restore options hidden.
        $form['field_restore_databases']['#type'] = 'hidden';
        $form['field_restore_webroot']['#type'] = 'hidden';
        $form['field_restore_files']['#type'] = 'hidden';
        $form['field_restore_settings_php']['#type'] = 'hidden';

      }
    }

    // Make deploy-status field hidden.
    $form['field_deploystatus'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';

    // Make logfile-field hidden.
    $form['field_logfile']['#type'] = 'hidden';

    // Make tag link field hidden.
    $form['field_tag_link']['#type'] = 'hidden';

    // Make restore id field hidden.
    $form['field_restore_id'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';

    // Make backups-ok field hidden.
    $form['field_backups_ok']['#type'] = 'hidden';

    // Make backups-deleted field hidden.
    $form['field_backups_deleted']['#type'] = 'hidden';
  }

  // Predefine project / other stuff for new environment.
  if (isset($form['type']['#value']) && $form['type']['#value'] == 'environment') {

    // Display public SSH key.
    if (!$rsa_keyfile = variable_get('deployer_rsa_keyfile')) {
      drupal_set_messag(t('No SSH keyfile configured'));
    }
    if (!$pubkey = file_get_contents($rsa_keyfile . '.pub')) {
      return drupal_set_message(t('Cannot read SSH public key!'));
    }
    drupal_set_message(t('My public SSH key: <br/>%key', array('%key' => $pubkey)));

    // Alter environment.
    if (isset($_GET['project']) && !empty($_GET['project']) && is_numeric($_GET['project']) && $project = node_load($_GET['project'])) {
      // Set default project value.
      $form['field_project'][LANGUAGE_NONE]['#default_value'] = $_GET['project'];
    }

    // Get drush options instead of text field.
    $form['field_drush_version'][LANGUAGE_NONE][0]['value']['#type'] = 'select';
    $form['field_drush_version'][LANGUAGE_NONE][0]['value']['#options'] = _deployer_get_drush_options();
    unset($form['field_drush_version'][LANGUAGE_NONE][0]['value']['#maxlength']);
    unset($form['field_drush_version'][LANGUAGE_NONE][0]['value']['#size']);

    // Hide health info and drupal version field.
    $form['field_health_info'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';
    $form['field_drupal_version'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';
    $form['field_last_checked'][LANGUAGE_NONE][0]['value']['#type'] = 'hidden';

    // Conditional fields.
    // Define switch on which to act.
    $switch = array(
      'select[name="field_connection_type[und]"]' => array('value' => 'ssh'),
    );

    // Set states for fields.
    foreach ($form['field_server_instance']['und'] as $key => &$element) {
      if (is_numeric($key)) {
        // Active mode, used for ftp.
        $element['field_active_mode']['und']['#states'] = array(
          'invisible' => $switch,
        );
        // Site instances, used for ftp.
        $element['field_site_instance']['#states'] = array(
          'invisible' => $switch,
          'optional' => $switch,
        );
        // Password, used for ftp.
        $element['field_password']['und'][0]['value']['#states'] = array(
          'invisible' => $switch,
        );

        // Owner (group), used for ssh.
        $element['field_owner_group']['und'][0]['value']['#states'] = array(
          'visible' => $switch,
          'required' => $switch,
        );
        // Owner (user), used for ssh.
        $element['field_owner_user']['und'][0]['value']['#states'] = array(
          'visible' => $switch,
          'required' => $switch,
        );
      }
    }
  }

  // Predefine client for a new project.
  // Hide fields from project form.
  if (isset($form['type']['#value']) && $form['type']['#value'] == 'project') {
    // Alter project.
    if (isset($_GET['client']) && !empty($_GET['client']) && is_numeric($_GET['client']) && $project = node_load($_GET['client'])) {
      // Set default client value.
      $form['field_client'][LANGUAGE_NONE]['#default_value'][0] = $_GET['client'];
    }

    // Hide field_repo_path.
    $form['field_repo_path']['#type'] = 'hidden';
  }

  // Check for password fields.
  // Only save those if they are not empty.
  deployer_form_check_password_fields($form, $form_state);
}

/**
 * Implements hook_node_validate().
 *
 * Make sure only 1 deploy is running on an environment
 * at the same time.
 */
function deployer_node_validate($node, $form, &$form_state) {
  if ($node->type == 'deploy') {
    if (isset($form_state['input']['field_environment_id']['und']) && !empty($form_state['input']['field_environment_id']['und'])) {

      // Only act on saving deploy node. Not on deleting.
      if ($form_state['clicked_button']['#submit'][0] == 'node_form_submit') {
        $env = $form_state['input']['field_environment_id']['und'];
        $query = db_select('node', 'n');
        $query->join('field_data_field_environment_id', 'env', 'n.nid = env.entity_id');
        $query->join('field_data_field_deploystatus', 'dstat', 'n.nid = dstat.entity_id');

        $query->fields('n', array('nid'));

        $query->condition('n.type', 'deploy');
        $query->condition('dstat.field_deploystatus_value', 0.5);
        $query->condition('env.field_environment_id_target_id', $env);
        $result = $query->execute();

        $found = 0;
        foreach ($result as $row) {
          $found = 1;
        }

        if ($found) {
          form_set_error('Environment occupied', t('This environment is already occupied by another deploy! Please try again later.'));
        }
      }
    }
  }
}

/**
 * Check/fix form for password fields (type == '#deployer_password').
 *
 * Hide the password from the field, and only save
 * it if it's not empty.
 *
 * @param array $form
 *   Form array.
 * @param array $form_state
 *   Form state array.
 * @param string $field_name
 *   Field name. Only used in recursion.
 * @param array $parent_element
 *   Parent form array. Only used in recursion.
 */
function deployer_form_check_password_fields(&$form, &$form_state, $field_name = '', &$parent_element = array()) {
  if (is_array($form)) {
    if (isset($form['#deployer_password']) && $form['#deployer_password']) {
      // Protect this form element!
      if (isset($form_state['input']['op']) && empty($form_state['input'][$field_name])) {
        // Form submitted and password field is empty..
        unset($parent_element[$field_name]);
        return;
      }
      $form['#default_value'] = '';
    }

    foreach ($form as $key => &$element) {
      if (is_array($element)) {
        deployer_form_check_password_fields($element, $form_state, $key, $form);
      }
    }
  }
}
